/*===================================
		Copyright Yakibomb
*///=================================

// Later in the bootup, the functions starting with "CBaseEntity_" are converted to extend from CBaseEntity class.

/*===================================
//	AddVSCond
	- condition == only valid conditions that were compiled by the script,
	- duration == how long before the vscond expires. Default is infinite (-1),
	- provider == Who gave the condition. Default provider is the entity given the condition, unless you input null (if which case, becomes worldspawn)
*///=================================
function CBaseEntity_AddVSCond(condition=0,duration=-1,provider=worldspawn) {
	if ( VSCONDS_IsCondNumInvalid(condition,this) ) return false;
	
	condition = VSCONDS_ClampCondValue(condition);
	local condition_func = VSCONDS_GetCondFunc(condition);
	
	if ( !(condition_func.settingsBits & VSCONDS_BASE_ADDCOND_IGNORE_DEATH)
	  && VSCONDS_IsEntDead(this) ) return false;
	
	local ent_data_table = format("%s_%d", "VSCOND_TABLE", this.entindex());
	local initialized = true;
	
	VSCONDS_CondEntInit(this);
	
	if ( !( condition in ROOT[ent_data_table] ) )
		initialized = false;
	
	if ( duration < 0.0 )
		duration = -1;
	else
		duration = Time() + duration.tofloat();
	
	ROOT[ent_data_table][condition] <- duration;
	
	if ( !initialized ) {
		condition_func["constructor"](true,this,provider);
		condition_func["AddCond"]();
	}

	return true;
}

/*===================================
//	InVSCond
	- condition == only valid conditions that were compiled by the script
*///=================================
function CBaseEntity_InVSCond(condition) {
	if ( VSCONDS_IsCondNumInvalid(condition,this) ) return false;
	condition = VSCONDS_ClampCondValue(condition);
	
//	local cond_name = VSCOND_COND_NAMES[condition];
	local ent_data_table = format("%s_%d", "VSCOND_TABLE", this.entindex());
	if ( !( VSCOND_ENTS_UNDER_CONDS.find(ent_data_table) ) )
		return false;
	if ( !( condition in ROOT[ent_data_table] ) )
		return false;
	
	VSCONDS_CondEntInit(this)
	
	local duration = ROOT[ent_data_table][condition];
	
	if ( duration == -1 ) return true;
	
	return duration < Time() ? false : true
}
/*===================================
//	RemoveVSCond
	- condition == only valid conditions that were compiled by the script
	- alwaysRemove == Overrides "removeByCommand" set by author of conditions ( if you ever need it ) ( removeByCommand is set true by default )
*///=================================

function CBaseEntity_RemoveVSCond(condition, alwaysRemove=false) {
	if ( VSCONDS_IsCondNumInvalid(condition,this) ) return false;
	condition = VSCONDS_ClampCondValue(condition);
	
//If command has `removedByCommand == false` defined in class, don't even try to remove the condition
	if ( alwaysRemove ) {
	// checks bit VSCONDS_BASE_REMOVEABLE_BY_COMMAND in condition's settings
		if ( !(VSCOND_COND_FUNC[condition]["settingsBits"] & VSCONDS_BASE_REMOVEABLE_BY_COMMAND) )
			return false;
	}

	local ent_data_table = format("%s_%d", "VSCOND_TABLE", this.entindex());
// Validates if condition exists, and creates it if not
	VSCONDS_CondEntInit(this);
	
// If condition didn't exist beforehand, do nothing
	local ent_data = ROOT[ent_data_table]
	if ( !( condition in ent_data ) )
		return false;
	
	local ent = this.entindex();
	local condition_func = VSCONDS_GetCondFunc(condition);
	condition_func["constructor"](false,this);
	condition_func["RemoveCond"]();
	
	VSCONDS_RemoveAttachmentEnt(ent,condition)
	
	if ( condition_func.use_think_script == true ) {
		VSCONDS_RemoveThinkToCond(ent,condition);
	}
	
	if ( ent_data.len() == 2 ) {
		if ( this in VSCOND_HIDDEN_WEARABLES ) {
			VSCOND_HIDDEN_WEARABLES[this].Kill();
			delete VSCOND_HIDDEN_WEARABLES[this];
		}
	}
	
	delete ent_data[condition];
	
	return true;
}

function CBaseEntity_GetVSCondDuration(condition) {
	if ( VSCONDS_IsCondNumInvalid(condition,this) ) return 0.0;
	condition = VSCONDS_ClampCondValue(condition);
	
	local ent_data_table = format("%s_%d", "VSCOND_TABLE", this.entindex());
	if ( !( VSCOND_ENTS_UNDER_CONDS.find(ent_data_table) ) )
		return 0.0;
	if ( !( condition in ROOT[ent_data_table] ) )
		return 0.0;
	
	local duration = ROOT[ent_data_table][condition];
	local time = Time();
	
	return  duration > time ? duration - time : ( duration == -1 ? 1e30 : 0 );
}



/*===================================
		Copyright Yakibomb
*///=================================