/*===================================
		Copyright Yakibomb
*///=================================

/*===================================
	- Validates current entity vs entity the condition was originally applied to
	- If both are the same, continue to check their condition, otherwise branch off and delete them
	- In this way we don't give conditions to entities that weren't intended to it
*///=================================
function VSConds_Think_Validate_Entity(ent, ent_datatable) {
	
	if ( ent_datatable.len() == 2 ) { }
	else {
		local ent_validate = ent_datatable[VSCond_MAX_Conditions + 1];
		ent_validate = VSCONDS_IsEntValid(ent_validate);
		if ( !ent_validate ) return true;
		if ( VSCONDS_IsEntValid(ent) == ent_validate ) return false;
	}
	
	return true;
}

/*===================================
	- Delete all condition data if entity no longer exists
*///=================================
function VSConds_Think_Delete(ent_data, ent_table_key) {
	local ent_index = split(ent_data.tostring(),"_");
	ent_index = ent_index[2].tointeger();
	
	VSCOND_ENTS_UNDER_CONDS.remove(ent_table_key);
	delete ROOT[ent_data];
	VSCONDS_RemoveThinkToCond_ALL(ent_index);
	VSCONDS_RemoveAttachmentEnt_ALL(ent_index)
}

/*===================================
	- checks states of entity, then sets and returns bits to be checked per condition
	In this way we only check state of entity once
*///=================================
function VSConds_Think_ChecksPerEntity(ent, time, ent_is_player) {
	local bits = 0;
	
	if ( VSCONDS_IsEntDead(ent) ) {	//checks if entity is not alive
		bits = bits | VSCONDS_BASE_REMOVEABLE_BY_DEATH;
	}
	
	if ( ent_is_player ) {
		if ( ent.GetNextChangeClassTime() >= time )		//checks if player changed classes recently
			bits = bits | VSCONDS_BASE_REMOVEABLE_BY_CLASSCHANGE;
		if ( ent.GetNextChangeTeamTime() >= time )		//checks if player changed teams recently
			bits = bits | VSCONDS_BASE_REMOVEABLE_BY_TEAMCHANGE;
		if ( ent.GetNextRegenTime() >= time ) 			//checks if player was resupplied recently
			bits = bits | VSCONDS_BASE_REMOVEABLE_BY_RESUPPLY;
	}
	
	return bits;
}

/*===================================
	- checks condition settings vs entity states, removes condition if any are true
*///=================================
function VSConds_Think_ChecksPerCondition(ent_bits, cond_data, ent_is_player) {
	local cond_data_bits = cond_data.settingsBits;
	if ( ent_bits & VSCONDS_BASE_REMOVEABLE_BY_DEATH ) {
		if ( cond_data_bits & VSCONDS_BASE_REMOVEABLE_BY_DEATH ) {
			return true;
		}
	}
	
	if ( ent_is_player ) {
	
		if ( ent_bits & VSCONDS_BASE_REMOVEABLE_BY_CLASSCHANGE ) {
			if ( cond_data_bits & VSCONDS_BASE_REMOVEABLE_BY_CLASSCHANGE )	//checks if condition cannot be removed via class change
				return true;
		}
				
		if ( ent_bits & VSCONDS_BASE_REMOVEABLE_BY_TEAMCHANGE ) {
			if ( cond_data_bits & VSCONDS_BASE_REMOVEABLE_BY_TEAMCHANGE )	//checks if condition cannot be removed via team change
				return true;
		}
				
		if ( ent_bits & VSCONDS_BASE_REMOVEABLE_BY_RESUPPLY ) {
			if ( cond_data_bits & VSCONDS_BASE_REMOVEABLE_BY_RESUPPLY ) //checks if condition cannot be removed via resupply
				return true;
		}

	}
	
	return false;
}



/*===================================
	- The Big Think
*///=================================
function VSConds_Think() {
	if ( VSCOND_ENTS_UNDER_CONDS.len() == 1 ) {
	//	printl("I sleep"); 
		return VSCond_Think_SleepTime;
	}
	//printl("I run");
	
	foreach( ent_table_key,ent_data in VSCOND_ENTS_UNDER_CONDS )
	{
	// Validates current entity vs entity the condition was originally applied to
		if ( ent_data == null ) continue
		
		local ent_data_string = ent_data;
		ent_data = ROOT[ent_data_string];
		local ent = ent_data[VSCond_MAX_Conditions];
		
		if ( VSConds_Think_Validate_Entity(ent, ent_data) ) {
			VSConds_Think_Delete(ent_data_string, ent_table_key);
			break;
		}
		
	// Since we know the entity exists/IsValid, check duration and if target is dead/changed classes
	
	// Sets up vars so we don't do that every time a condition is checked
		local time = Time();
		local ent_is_player = ent.IsPlayer();
		local ent_bits = VSConds_Think_ChecksPerEntity(ent, time, ent_is_player);
		
	// Checks each condition that the entity has on them
		foreach (condition,duration in ent_data ) {
			if ( condition >= VSCond_MAX_Conditions ) continue
			local cond_data = VSCOND_COND_FUNC[condition];
			if ( VSConds_Think_ChecksPerCondition(ent_bits, cond_data, ent_is_player) ) {
				ent.RemoveVSCond(condition, true);	//`true` means it always removes condition, no matter settings
			}
			else if ( duration == -1 ) { } 
			else if ( duration <= time ) { ent.RemoveVSCond(condition, true); }
		}
	}
	return VSCond_Think_RunTime;
}